/*    
******************************************************************************
	wd90c00 (PVGA1B, wd90c90) IO functions and variables
******************************************************************************

   Copyright (C) 1997 by Brian S. Julin, <bri@calyx.com>

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
******************************************************************************
*/

#define _wd90c00_io_c

/* MAINTAINER	Brian S. Julin, <bri@calyx.com> */

#include <WD/wd90c00_io.h>
#include "pvga1a_io.c"

/**** CRT Controller Register Manipulation ****/

void save_CRT(kgiu8 * buf)   
{
  register kgiu8 i; 
  int lockval1;
  lockval1 = runlockCRT();
  for (i = 0; i < NoCRTRegs; i++) 
    {
      *(buf++) = inbCRT(i);
    }
  outbCRT(lockval1, CRT_UPCRT_LOCK); /* Restore CRT lock */
}

void restore_CRT(kgiu8 *buf)  /* wd90c00+ has more locks */
{
  register kgiu8 i;
  int lockval1, lockval2;

  lockval1 = rwunlockCRT();
  
  outbCRT(inbCRT(CRT_HBLANKEND) | CR03_IS_VGA,CRT_HBLANKEND);
  
  for (i = 0; i < NoCRTRegs; i++) 
    {
      switch (i) 
	{
	case CRT_HBLANKEND:
	  *buf |= CR03_IS_VGA;
	  break;
	case CRT_VSYNCEND:
	  *buf &= ~CR11_LOCKTIMING;
	  break;
	default:
	  break;
	}
      outbCRT(*(buf++), i);
    }

  outbCRT(((lockval1 & 0xff00) >> 8 ), CRT_UPCRT_LOCK); /* Restore CRT lock */
  lockval2 = wunlockGRC();                       /* Unlock locking register */
                                   /* Restore locks on CRT timing registers */
  outbGRC(lockval1 & 0xff, GRC_CRTC_CTL); 
  outbGRC(lockval2, GRC_UPGRC_LOCK);                    /* Restore GRC lock */

}

#include "pvga1a_io.c"








