/*
 * GWT - General Windowing Toolkit
 *
 * Copyright (C) 1998 MenTaLguY - mentalg@geocities.com
 *                    Rodolphe Ortalo - ortalo@laas.fr
 *                    Tristan Wibberley - bloater@ps.cus.umist.ac.uk
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: geometry.c,v $
 * Revision 1.10  2000/03/19 19:32:46  ortalo
 * Some debugging and modifications (wrt OpenAmulet mainly).
 *
 * Revision 1.9  2000/03/13 22:02:55  ortalo
 * Committed new version of LibGWT corresponding to 0.0.2.
 * The event management API is probably more or less defined. The simplest
 * approach finally won. (I should have remembered: KISS!)
 *
 * Revision 1.8  1999/05/20 20:34:23  ortalo
 * Several bugs corrections and small improvements.
 *
 * Revision 1.7  1999/05/13 20:01:38  ortalo
 * New version of libgwt. Now using LibGII for input management.
 * Some drawing functions implemented. Visibility management should
 * be rather okay. Still some work to do, but now, your eyes can
 * look at some drawings... not only code. ;-)
 *
 * Revision 1.6  1998/11/17 15:13:28  tristan
 *
 * Changed my email address
 *
 * Revision 1.5  1998/09/02 02:21:44  tristan
 * Loadsa great stuff done. Visible/invalid regions,
 *  copy on move - fixed tt_region.c too!
 *
 * Revision 1.4  1998/08/23 18:57:09  ortalo
 * Minor editing (mainly comments).
 *
 * Revision 1.3  1998/08/16 18:40:54  ortalo
 * A new version, STILL IN VERY ALPHA STATE... (Be patient.)
 * (However, this version is a little better... :-)
 *
 * Revision 1.2  1998/08/11 22:10:06  ortalo
 * Incorporated the changes made by Tristan Wibberley. The code is
 * still in alpha development state.
 *
 * Revision 1.1  1998/07/09 18:38:39  ortalo
 * Initial integration of libgwt in the GGI repository.
 * This is experimental code, for your eyes only !
 * (Don't execute... ;-)
 *
 */

#include "internal.h"
#include "gwt_debug.h"

/**********************************
 * Convenience geometry functions *
 **********************************/
/*
 * Translate a window and all its family (Recursive)
 */
static void gwt_translate(const gwt_window_t win, const int dx, const int dy)
{
  gwt_window_t child;
  /* translates the window origin */
  win->origin.x += dx;
  win->origin.y += dy;
  /* translates the window cursor position */
  win->ptr_x -= dx;
  win->ptr_y -= dy;
  /* translates the window geometry */
  gwt_region_move(win->geometry, dx, dy);
  /* iterates over all children */
  child = GWT_Z_BOTTOM_CHILD(win);
  while (child != NULL)
    {
      gwt_translate(child, dx, dy);
      child = GWT_Z_UP(child);
    }
  return;
}
/*
 * Finds the root window
 */
static gwt_window_t get_root(gwt_window_t win)
{
  gwt_window_t curwin = win; /* TODO: Use win directly ? */
  while (curwin->level > 0)
    {
      curwin = curwin->wparent;
    }
  return curwin;
}
/**************************************************************************
 *                    GWT PUBLIC EXPORTED FUNCTIONS                       *
 **************************************************************************/
/*
 * Set/Change a window geometry [in PARENT coordinates]
 */
/* TODO: Update the cursor position... ! */
int gwtSetGeometry(gwt_window_t win, gwt_region_t region)
{
  gwt_region_t tmp;
  gwt_region_create(&tmp);
  gwt_region_copy(tmp, region);
  gwt_region_move(tmp, win->origin.x, win->origin.y);
  /* Changes the window geometry */
  gwt_region_copy(win->geometry, tmp);
  gwt_region_destroy(tmp);
  /* Update visibility and co. */
  gwt_propagate_geometry_update(win);
  /* TODO: Do intelligent invalid update */
  /* TODO: We can update ourselves some of the invalid region by
     TODO: mere copying... We should.
  */
  return 0;
}

/*
 * Resize a window - NB: Turn it into a rectangle !!!
 */
int gwtResize(gwt_window_t win, int w, int h)
{
  /* Changes the window geometry */
  gwt_region_init(win->geometry, win->origin.x, win->origin.y,
		  win->origin.x + w, win->origin.y + h);
  /* Update visibility and co. */
  gwt_propagate_geometry_update(win);
  /* There is nothing we can update easily on screen */
  /* TODO: Does a rough update, change that */
  if (!gwtIsHidden(win))
    gwt_invalidate_all(win->wparent);
  return 0;
}
/*
 * Move a window to (x,y) [in PARENT coordinates]
 */
int gwtMove(gwt_window_t win, int x, int y)
{
  gwt_region_t invalidated;
  /* TODO: return if root window !!! -- ortalo */ 
  /* Computes the delta */
  int dx = x - (win->origin.x - win->wparent->origin.x); /* wrt. parent coord.*/
  int dy = y - (win->origin.y - win->wparent->origin.y);
  /* Remembers the old potentially visible region */
  gwt_region_create(&invalidated);
  gwt_region_copy(invalidated, win->potentially_visible);
  /* Translate the window geometry and its children */
  gwt_translate(win, dx, dy);
  /* Update visibility and co. */
  gwt_propagate_geometry_update(win);
  /* Invalidates some areas */
  gwt_region_difference(invalidated, win->potentially_visible);
  gwt_invalidate(win->wparent, invalidated);
  /*
   * TODO: Add something invalid in our own window ?!?
   */
  gwt_invalidate_all(win); /* TODO: Check if useful */
  /* TODO: We must update ourselves some of the modified region by
     TODO: mere copying... We should: currently the moved region
     TODO: is not yet really moved.
  */
  gwt_region_destroy(invalidated);
  return 0;
}
/*
 * Provide access to the main window regions
 */
int gwtGetGeometry(gwt_window_t win, gwt_region_t region) {
  gwt_region_copy(region, win->geometry);
  if (win->level != 0) {
    gwt_region_move(region, -win->origin.x, -win->origin.y);
  }
  return 0;
}
int gwtGetInvalidRegion(gwt_window_t win, gwt_region_t region) {
  gwt_region_copy(region, win->invalid);
  if (win->level != 0) {
    gwt_region_move(region, -win->origin.x, -win->origin.y);
  }
  return 0;
}
int gwtGetVisibleRegion(gwt_window_t win, gwt_region_t region) {
  gwt_region_copy(region, win->visible);
  if (win->level != 0) {
    gwt_region_move(region, -win->origin.x, -win->origin.y);
  }
  return 0;
}
/*
 * Allows to translate coordinates between two windows
 */
int gwtTranslateCoordinates(gwt_window_t source, int sx, int sy,
			    gwt_window_t destination, int *dx, int *dy)
{
  /* TODO: Check that the root is identical */
  if (get_root(source) == get_root(destination))
    {
      (*dx) = sx + source->origin.x - destination->origin.x;
      (*dy) = sy + source->origin.y - destination->origin.y;
      return 0;
    }
  else
    {
      return -1;
    }
  LIBGWT_ASSERT(0,"UNREACHABLE CODE!");
}
